@@ -2,12 +2,12 @@ import time, json, sys, subprocess
 
 def catch_error_and_exit(errmsg, logger):
 	logger.error(errmsg)
-	sys.exit(errmsg)
+	sys.exit("stackql-deploy operation failed 🚫")
 
 def get_type(resource, logger):
     type = resource.get('type', 'resource')
-    if type not in ['resource', 'query', 'script']:
-        catch_error_and_exit(f"resource type must be 'resource', 'script' or 'query', got '{type}'", logger)
+    if type not in ['resource', 'query', 'script', 'multi']:
+        catch_error_and_exit(f"resource type must be 'resource', 'script', 'multi' or 'query', got '{type}'", logger)
     else:
         return type
 
@@ -66,39 +66,51 @@ def run_stackql_query(query, stackql, suppress_errors, logger, retries=0, delay=
 
 def error_detected(result):
     """parse stdout for known error conditions"""
-    # aws cloud control hack...
     if result['message'].startswith('http response status code: 4') or result['message'].startswith('http response status code: 5'):
         return True
     if result['message'].startswith('error:'):
         return True
     if result['message'].startswith('disparity in fields to insert and supplied data'):
-            return True    
+        return True
+    if result['message'].startswith('cannot find matching operation'):
+        return True    
     return False
-        
-def run_stackql_command(command, stackql, logger):
-    try:
-        logger.debug(f"(utils.run_stackql_command) executing stackql command:\n\n{command}\n")
-        result = stackql.executeStmt(command)
-        logger.debug(f"(utils.run_stackql_command) stackql command result:\n\n{result}, type: {type(result)}\n")
 
-        if isinstance(result, dict):
-            # If the result contains a message, it means the execution was successful
-            if 'message' in result:
-                if error_detected(result):
-                    catch_error_and_exit(f"(utils.run_stackql_command) error occurred during stackql command execution:\n\n{result['message']}\n", logger)
-                logger.debug(f"(utils.run_stackql_command) stackql command executed successfully:\n\n{result['message']}\n")
-                return result['message'].rstrip()
-            elif 'error' in result:
-                # Check if the result contains an error message
-                error_message = result['error'].rstrip()
-                catch_error_and_exit(f"(utils.run_stackql_command) error occurred during stackql command execution:\n\n{error_message}\n", logger)
+def run_stackql_command(command, stackql, logger, ignore_errors=False, retries=0, retry_delay=5):
+    attempt = 0
+    while attempt <= retries:
+        try:
+            logger.debug(f"(utils.run_stackql_command) executing stackql command (attempt {attempt + 1}):\n\n{command}\n")
+            result = stackql.executeStmt(command)
+            logger.debug(f"(utils.run_stackql_command) stackql command result:\n\n{result}, type: {type(result)}\n")
+
+            if isinstance(result, dict):
+                # If the result contains a message, it means the execution was successful
+                if 'message' in result:
+                    if not ignore_errors and error_detected(result):
+                        if attempt < retries:
+                            logger.warning(f"dependent resource(s) may not be ready, retrying in {retry_delay} seconds (attempt {attempt + 1} of {retries + 1})...")
+                            time.sleep(retry_delay)
+                            attempt += 1
+                            continue  # Retry the command
+                        else:
+                            catch_error_and_exit(f"(utils.run_stackql_command) error occurred during stackql command execution:\n\n{result['message']}\n", logger)
+                    logger.debug(f"(utils.run_stackql_command) stackql command executed successfully:\n\n{result['message']}\n")
+                    return result['message'].rstrip()
+                elif 'error' in result:
+                    # Check if the result contains an error message
+                    error_message = result['error'].rstrip()
+                    catch_error_and_exit(f"(utils.run_stackql_command) error occurred during stackql command execution:\n\n{error_message}\n", logger)
+            
+            # If there's no 'error' or 'message', it's an unexpected result format
+            catch_error_and_exit("(utils.run_stackql_command) unexpected result format received from stackql execution.", logger)
         
-        # If there's no 'error' or 'message', it's an unexpected result format
-        catch_error_and_exit("(utils.run_stackql_command) unexpected result format received from stackql execution.", logger)
-    
-    except Exception as e:
-        # Log the exception exit
-        catch_error_and_exit(f"(utils.run_stackql_command) an exception occurred during stackql command execution:\n\n{str(e)}\n", logger)
+        except Exception as e:
+            # Log the exception and exit
+            catch_error_and_exit(f"(utils.run_stackql_command) an exception occurred during stackql command execution:\n\n{str(e)}\n", logger)
+        
+        # Increment attempt counter if not continuing the loop due to retry
+        attempt += 1
 
 def pull_providers(providers, stackql, logger):
     logger.debug(f"(utils.pull_providers) stackql run time info:\n\n{json.dumps(stackql.properties(), indent=2)}\n")
@@ -169,18 +181,21 @@ def perform_retries(resource, query, retries, delay, stackql, logger, delete_tes
     elapsed = time.time() - start_time  # Calculate total elapsed time
     return False
 
-def export_vars(self, resource, export, expected_exports, protected_exports):
-    for key in expected_exports:
-        if key not in export:
-            catch_error_and_exit(f"(utils.export_vars) exported key '{key}' not found in exports for {resource['name']}.", self.logger)
-
+def export_vars(self, resource, export, expected_exports, expected_exports_all_dicts, protected_exports):
+    for item in expected_exports:
+        # check if all items are dictionaries
+        if expected_exports_all_dicts:
+            if list(item.values())[0] not in export:
+                catch_error_and_exit(f"(utils.export_vars) exported item '{list(item.values())[0]}' not found in exports for {resource['name']}.", self.logger)
+        else:
+            if item not in export:
+                catch_error_and_exit(f"(utils.export_vars) exported item '{item}' not found in exports for {resource['name']}.", self.logger)
     for key, value in export.items():
         if key in protected_exports:
             mask = '*' * len(str(value))
-            self.logger.info(f"🔒  set protected variable [{key}] to [{mask}] in exports")
+            self.logger.info(f"🔒 set protected variable [{key}] to [{mask}] in exports")
         else:
-            self.logger.info(f"➡️  set [{key}] to [{value}] in exports")
-
+            self.logger.info(f"📤 set [{key}] to [{value}] in exports")
         self.global_context[key] = value  # Update global context with exported values
 
 def run_ext_script(cmd, logger, exports=None):
@@ -208,4 +223,18 @@ def run_ext_script(cmd, logger, exports=None):
         return exported_vars
     except json.JSONDecodeError:
         catch_error_and_exit(f"(utils.run_ext_script) external scripts must return a valid JSON object {result.stdout}", logger)
-        return None
\ No newline at end of file
+        return None
+
+def check_all_dicts(items, logger):
+    """ Check if all items(list) are of the same type (either all dicts or all non-dicts).
+    """
+    all_dicts = all(isinstance(item, dict) for item in items)
+    no_dicts = all(not isinstance(item, dict) for item in items)
+
+    if not all_dicts and not no_dicts:
+        catch_error_and_exit(f"type inconsistency: all items({items}) must be either dicts or non-dicts", logger)
+    
+    if all_dicts:
+        return True
+    else:
+        return False
